/*
   Miscellaneous functions
   --------------------------------------------------------------------
   VCHE NC - Virtual Console Hex Editor NCURSES version

   Copyright (C) 1998, 1999 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

static void print_long( long value, int prec);

int flag_bell;     /* Bell flag, set bell on and off               */
int flag_edit;     /* Edit flag                                    */
int flag_nohex;    /* No hex flag, toggle viewing the hex segment  */
int flag_readonly; /* Read Only flag, disable/enable readonly mode */

static char trans_buffer[ 32];

void do_exit( int status)
{
 term_unhide_cursor();
 endwin();
 exit( status);
}

void draw_filename( char *filename)
{
 char *ptr;
 int len;
 int mlen;

 ptr = rindex( filename, '/');

 if( ptr == NULL)
     ptr = filename;
 else
     ptr++;

 len  = strlen( ptr);
 mlen = COLS - 26;

 nc_color_puts( 0, 0, mlen, replicate( ' ', mlen), NC_COLOR_BACKCYAN);

 if( len > mlen) {
     nc_color_putc( mlen - 1, 0, ACS_RARROW, NC_COLOR_ATTENTION);
     len = mlen - 1;
 }

 nc_color_puts( 0, 0, len, ptr, NC_COLOR_BACKCYAN);
}

void draw_main_screen( void)
{
 nc_fill_screen( NC_CYANBLUE);

 nc_color_puts( 0, 0, COLS, replicate( ' ', COLS), NC_COLOR_BACKCYAN);

 nc_color_putc( COLS - 26, 0, ACS_VLINE, NC_COLOR_BACKCYAN);
 nc_color_puts( COLS - 25, 0, 6, "WXMEB0", NC_COLOR_BACKCYAN);
 nc_color_putc( COLS - 19, 0, ACS_VLINE, NC_COLOR_BACKCYAN);

 nc_color_putc( COLS - 5 , 0, ACS_VLINE, NC_COLOR_BACKCYAN);

 nc_color_putc( COLS - 1 , 0, '%', NC_COLOR_BACKCYAN);

 draw_fkeys();
}

void draw_fkeys( void)
{
 int sp, x, y;

 /*
  * I have done it for 132x60 and others non 80xANY modes
  */

 sp = ( COLS / 10) - 8;
 y = LINES - 1;

 nc_color_puts( 0, LINES - 1, COLS, replicate( ' ', COLS), NC_COLOR_BACKCYAN);

 x  = 2;      nc_color_puts( x, y, 4, "HELP"  , NC_COLOR_BACKCYAN); /* F1  */
 x += 8 + sp; nc_color_puts( x, y, 4, "EDIT"  , NC_COLOR_BACKCYAN); /* F2  */
 x += 8 + sp;
 if( flag_nohex)                      /* F3  */
     nc_color_puts( x, y, 5, "HEX  ", NC_COLOR_BACKCYAN); 
 else
     nc_color_puts( x, y, 5, "NOHEX", NC_COLOR_BACKCYAN);
 x += 8 + sp; nc_color_puts( x, y, 6, "SEARCH", NC_COLOR_BACKCYAN); /* F4  */
 x += 8 + sp; nc_color_puts( x, y, 6, "SAGAIN", NC_COLOR_BACKCYAN); /* F5  */
 x += 8 + sp; nc_color_puts( x, y, 4, "GOTO"  , NC_COLOR_BACKCYAN); /* F6  */
 x += 8 + sp; nc_color_puts( x, y, 5, "FILES" , NC_COLOR_BACKCYAN); /* F7  */
 x += 8 + sp; nc_color_puts( x, y, 6, "ATABLE", NC_COLOR_BACKCYAN); /* F8  */
 x += 8 + sp; nc_color_puts( x, y, 6, "GNUGPL", NC_COLOR_BACKCYAN); /* F9  */
 x += 8 + sp; nc_color_puts( x, y, 4, "QUIT"  , NC_COLOR_BACKCYAN); /* F10 */

 x  = 0;      nc_color_puts( x, y, 2, " 1", NC_LIGHTGRAY );
 x += 8 + sp; nc_color_puts( x, y, 2, " 2", NC_LIGHTGRAY );
 x += 8 + sp; nc_color_puts( x, y, 2, " 3", NC_LIGHTGRAY );
 x += 8 + sp; nc_color_puts( x, y, 2, " 4", NC_LIGHTGRAY );
 x += 8 + sp; nc_color_puts( x, y, 2, " 5", NC_LIGHTGRAY );
 x += 8 + sp; nc_color_puts( x, y, 2, " 6", NC_LIGHTGRAY );
 x += 8 + sp; nc_color_puts( x, y, 2, " 7", NC_LIGHTGRAY );
 x += 8 + sp; nc_color_puts( x, y, 2, " 8", NC_LIGHTGRAY );
 x += 8 + sp; nc_color_puts( x, y, 2, " 9", NC_LIGHTGRAY );
 x += 8 + sp; nc_color_puts( x, y, 2, "10", NC_LIGHTGRAY );
}

void draw_offset_n_perc( long offset, long filesize)
{
 print_long( offset * 100 / filesize, 3);
 nc_color_puts( COLS -  4, 0,  3, trans_buffer, NC_COLOR_BACKCYAN);

 print_long( offset, 13);
 nc_color_puts( COLS - 18, 0, 13, trans_buffer, NC_COLOR_BACKCYAN);
}

void print_long( long value, int prec)
{
 register char *bp = trans_buffer + prec;

 /* Yes I have knowledge of the existence of the printf family of
    functions, but I don't want to call them! they are slow, and
    this part of the program gets called too much time, I do this
    because I hate slow programs!, please use gprof and time to 
    measure your programs.
 */

 do {
     *--bp = hex_trans[ value % 10];
 } while( (value /= 10) != 0);

 /* space padding */
 while( bp != trans_buffer)
        *--bp = ' ';
}


void set_flag( int flag)
{
 int xsflag = COLS - 25;

 switch( flag) {
         case FLAG_BELL: if( flag_bell)
                             nc_color_putc( xsflag + 4, 0, 'B' , NC_COLOR_ATTENTION);
                         else
                             nc_color_putc( xsflag + 4, 0, 'B' , NC_COLOR_BACKCYAN);
                         break;
         case FLAG_EDIT: if( flag_edit)
                             nc_color_putc( xsflag + 3, 0, 'E' , NC_COLOR_ATTENTION);
                         else
                             nc_color_putc( xsflag + 3, 0, 'E' , NC_COLOR_BACKCYAN);
                         break;
         case FLAG_NOHEX:
                         if( flag_nohex)
                             nc_color_putc( xsflag + 1, 0, 'X' , NC_COLOR_ATTENTION);
                         else
                             nc_color_putc( xsflag + 1, 0, 'X' , NC_COLOR_BACKCYAN);
                         break;
         case FLAG_READONLY:
                         if( flag_readonly)
                             nc_color_putc( xsflag, 0, 'W' , NC_COLOR_ATTENTION);
                         else
                             nc_color_putc( xsflag, 0, 'W' , NC_COLOR_BACKCYAN);
                         break;
 }
}

void *allocate_mem( int size)
{
 void *voidptr = NULL;

 voidptr = calloc( 1, size + 1);

 if( voidptr == NULL) {
     fprintf( stderr, "Not enough memory\n");
     do_exit( 1);
 }

 return voidptr;
}

char *trim_spaces( char *s)
{
 static char a[ 8192];
 char *save = s;
 char *start, *end;

 start = s;
 end = strchr( s, 0);

 if( *end == '\n' || *end == '\r') *end = 0;

 while( end >= start && *end <= ' ') end--;
 *++end = 0;

 while( end >= start && *start <= ' ') start++;
 save = start;

 memset( &a, 0, sizeof( a));
 strcpy( a, save);

 return a;
}

void set_hexmode_variables( void)
{

 bytes_per_row = COLS - 12;

 if( !flag_nohex) {
     bytes_per_row     = bytes_per_row * 4 / 17;
     bytes_per_row_hex = bytes_per_row * 13 / 4;
 }

 bytes_to_read = bytes_per_row * (LINES - 2);
}

int check_readonly( void)
{
 int i;
 if( !flag_readonly) return 1;

 term_beep();
 i = nc_ask( "We are in READ ONLY mode do you want to set READ/WRITE mode?", " Yes ", " No ", " Cancel ", 1);

 term_hide_cursor();

 if( i == 0) return 1;

 return -1;
}

char *replicate( char c, int size)
{
 static char rstr[ 2048];

 memset( &rstr, c, size);

 return rstr;
}

void translate_buffer_colors( int what_attr, attr_t *attr, short *color)
{
 switch( what_attr) {
         case BUF_BBLUE : *attr  = A_BOLD;
                          *color = 25;
                          break;
         case BUF_BLGRAY: *attr  = A_NORMAL;
                          *color = 26;
                          break;
         case BUF_WHITE : *attr  = A_BOLD;
                          *color = 21;
                          break;
 }
}

