/*
   Search functions
   --------------------------------------------------------------------
   VCHE NC - Virtual Console Hex Editor NCURSES version

   Copyright (C) 1998, 1999 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

char srch[ 64]; /* The string to search */
static long search_again_offset;
static long re_search;

static void search_string( char *str, long offset);
static long search_mem( char *sbuffer, char *pattern, long sizes, long sizep);
static void search_set_char_colorxy( chtype byte, int what_attr);

void search( void)
{
 WINDOW *w;
 char temp[ 64];
 char rever[ 64];
 int b1, b2;
 int len;
 int i;

 w = nc_dialog_box( "Enter search string: ", srch, 38, NC_COLOR_BACKCYAN, NC_WHITE);

 term_wgoto( w, 2, 2);

 key_scanf( w, srch, 38);

 strcpy( temp, trim_spaces( srch));

 i = 0;
 len = strlen( temp);

 if( !strncasecmp( temp, "0x", 2)) {
     while( len > 0) {
            temp[ len - 2] = toupper( temp[ len - 2]);
            temp[ len - 1] = toupper( temp[ len - 1]);
            b1 = b162bin( temp, len - 2);
            b2 = b162bin( temp, len - 1);
            rever[ i] = b1 * 16 + b2;
            len-=2;
            i++;
     }
     rever[ i] = 0;
     len = strlen( rever);
     i = 0;
     while( len > 0) {
            temp[ i] = rever[ len - 1];
            len--;
            i++;
     }
     temp[ i] = 0;
 } /* end if strncase... */

 memset( &srch, 0, 63);
 strcpy( srch, temp);

 ll_edit_head = NULL;

 if( srch[ 0] != 0 )
     search_string( srch, 0);

 delwin( w);
}

void search_again( void)
{
 search_again_offset = search_again_offset < fpos? fpos : search_again_offset;

 ll_edit_head = NULL;

 if( srch[ 0] != 0 )
     search_string( srch, search_again_offset);
}

void filegoto( void)
{
 WINDOW *w;
 char posstr[ 64];
 long newpos = -1;

 memset( &posstr, 0, sizeof( posstr));

 sprintf( posstr, "%ld", fpos);

 w = nc_dialog_box( "Enter file position: ", posstr, 38, NC_COLOR_BACKCYAN, NC_WHITE);

 term_wgoto( w, 2, 2);
 key_scanf( w, posstr, 38);

 if( !strncasecmp( posstr, "0x", 2)) {
     sscanf( posstr, "%lx", &newpos);
 }
 else
     newpos = atol( posstr);

 if( newpos < 0 || newpos > stbuf.st_size) {
     nc_show_alert_box( "Invalid offset!", 5);
     delwin( w);
     return;
 }

 fpos = lseek( fd, (long) newpos, SEEK_SET);

 delwin( w);
}

static void search_string( char *str, long offset)
{
 WINDOW *w;
 struct ll_edit *node;
 struct ll_edit *tmp;
 char search_buf[ 8192];
 long filepos = 0;
 long bytes_read;
 long sizep = strlen( str);
 long retv;
 int i = 0;

 search_again_offset = 0;
 lseek( fd, (long) offset, SEEK_SET);

 w = nc_show_message( " Searching... ", 5);

 while( filepos < stbuf.st_size) {
        filepos = file_getpos();
        bytes_read = read( fd, &search_buf, sizeof( search_buf) - 1);
        if( bytes_read <= 0) 
            break;

        retv = search_mem( search_buf, str, bytes_read, sizep);

        if( retv >= 0) {
            fpos = filepos + retv;
            search_again_offset = fpos + sizep;
            while( i < sizep) {
                   node = ll_edit_head;
                   tmp  = ll_edit_head;

                   while( node != NULL) {
                          if( node->filepos < ( fpos + i)) {
                              tmp  = node;
                              node = node->next;
                          } else
                              break;
                   }

                   ll_edit_add( search_buf[ retv + i], fpos + i, tmp);
                   i++;
            }
            delwin( w);
            lseek( fd, (long) fpos, SEEK_SET);
            return;
        }
        if( retv == -2)
            lseek( fd, (long) filepos + re_search, SEEK_SET);
 }

 nc_show_alert_box( "String not found", 5);

 delwin( w);
 lseek( fd, (long) fpos, SEEK_SET);
}

static long search_mem( char *sbuffer, char *pattern, long sizes, long sizep)
{
 long found = 0;
 int i, j;

 i = j = 0;

 while( j < sizes && i < sizep && found < sizep) {

        if( sbuffer[ j] == pattern[ i]) {
            found++; 
            j++;
            i++;
        } else {
            j++;
            i = 0;
            found = 0;
        }
 }

 if( found == sizep)
     return j - found;

 if( found > 0) {
     re_search = j - found;
     return -2;
 }

 return -1;
}

void search_set_color( void)
{
 struct ll_edit *node = ll_edit_head;
 int bytes;

 while( node != NULL) {
        if( node->filepos < fpos)
            node = node->next;
        else
            break;
 }

 if( node != NULL) {
     bytes = node->filepos - fpos;
     while( bytes < bread && node != NULL && node->filepos <= (fpos + bytes)) {

          ey = bytes / bytes_per_row;
          ex = - ey * bytes_per_row + bytes;

          if( flag_nohex)
              search_set_char_colorxy( node->byte, BUF_BLGRAY);
          else
              set_char_colorxy( node->byte, BUF_BLGRAY);

          node = node->next;
          ex++;
          if( ex > bytes_per_row - 1) {
              ex = 0;
              ey++;
          }
          if( node != NULL)
              bytes = node->filepos - fpos;
     }

 }

 ll_edit_free();
}

static void search_set_char_colorxy( chtype byte, int what_attr)
{
 int posa;
 int ay, ax;
 int posxya;

 ay = ey + 1;
 ax = ex + COLS - bytes_per_row;

 posxya = 4 + 2 *( ay * COLS + ax);
 posa = posxya - (4 + 2 * COLS);

 obuffer[ posa    ] = byte;
 obuffer[ posa + 1] = what_attr;
}

