/*
   Editor functions
   --------------------------------------------------------------------
   VCHE RAW - Virtual Console Hex Editor Run Anywhere version

   Copyright (C) 1998, 1999 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

extern char hex_trans_byte1[];
extern char hex_trans_byte2[];

extern char *hbuffer;

int edit_mask;        /* Edit mask for XOR/etc F* keys                  */
static char *cbuffer; /* Color buffer                                   */
static int tab_flag;  /* Switch from hex to ascii editing and viceversa
                         (i.e. left side = hex, right side = ascii edit */
int ex, ey;           /* X and Y coordinates                            */
static int hx, hy;    /* Hex side x, y                                  */
static int ax, ay;    /* Ascii side x, y                                */
static int f_digit;   /* First or second digit (for hex editing)        */
static long posxyh;   /* xy position on virtual screen (hex side)       */
static long posxya;   /* xy position on virtual screen (ascii side)     */

static void set_edit_fkeys( void);
static void set_all_xy( void);
static void edit_set_colorxy( int attr);
static void edit_the_buffer( char c);
static void edit_set_charxy_ascii( char c);
static void edit_set_charxy_hex( char c);
#define edit_set_color_buffer() edit_set_colorxy( cbuffer[ ey * bytes_per_row + ex])
#define edit_set_color_in_buffer( attr) cbuffer[ ey * bytes_per_row + ex] = attr
static void set_editchange_colors( void);
static void edit_set_mask( void);
static void edit_bitops( int bitop);
#define edit_xor() edit_bitops( EDIT_BITOP_XOR)
#define edit_or()  edit_bitops( EDIT_BITOP_OR )
#define edit_and() edit_bitops( EDIT_BITOP_AND)
#define edit_neg() edit_bitops( EDIT_BITOP_NEG)
#define edit_add() edit_bitops( EDIT_BITOP_ADD)
#define edit_sub() edit_bitops( EDIT_BITOP_SUB)
static int edit_ask_save( int pos);
static void edit_save_data( void);

void edit( void)
{
 char c;

 cbuffer = ( char *) allocate_mem( ( scrn.lines - 2 ) * scrn.cols );
 memset( cbuffer, VC_COLOR_BACKBLUE, ( scrn.lines - 2 ) * scrn.cols);

 ll_edit_head = NULL;

 ex = ey  = 0;
 f_digit  = 0;
 tab_flag = 0;

 hex_showfile();
 raw_flush();

 set_edit_fkeys();

 term_unhide_cursor();

 while( 1) {
        if( ex <  0) {
            ex = bytes_per_row - 1;
            ey--;
        }

        if( ex > bytes_per_row - 1) {
            ex = 0;
            ey++;
        }

        edit_set_colorxy( VC_WHITE);

        c = getc( stdin);

        if( c == '\t'){
            tab_flag = tab_flag ? 0 : 1;
            continue;
        }

        if( c == 27) {
            switch( key_process()) {
                    case VC_KEY_F2   : if( edit_ask_save( EDIT_ASK_SAVE) == -1) {
                                           edit_set_color_buffer();
                                           term_hide_cursor();
                                           free( cbuffer);
                                           return;
                                       }
                                       continue;
                    case VC_KEY_F3   : edit_set_mask();
                                       continue;
                    case VC_KEY_F4   : edit_xor(); 
                                       continue;
                    case VC_KEY_F5   : edit_or();
                                       continue;
                    case VC_KEY_F6   : edit_and();
                                       continue;
                    case VC_KEY_F7   : edit_neg();
                                       continue;
                    case VC_KEY_F8   : edit_add();
                                       continue;
                    case VC_KEY_F9   : edit_sub();
                                       continue;
                    case VC_KEY_F10  : if( edit_ask_save( EDIT_ASK_QUIT) == -1) {
                                           edit_set_color_buffer();
                                           term_hide_cursor();
                                           free( cbuffer);
                                           return;
                                       }
                                       continue;
                    case VC_KEY_UP   : edit_set_color_buffer();
                                       ey--;
                                       if( ey < 0) {
                                           ey = 0;
                                           file_prev_line();
                                           set_editchange_colors();
                                       }
                                       continue;
                    case VC_KEY_DOWN : edit_set_color_buffer();
                                       ey++;
                                       if( ey > scrn.lines - 3) {
                                           ey = scrn.lines - 3;
                                           file_next_line();
                                           set_editchange_colors();
                                       }
                                       continue;
                    case VC_KEY_RIGHT: if( !tab_flag) {
                                           if( f_digit) {
                                               edit_set_color_buffer();
                                               ex++;
                                           }
                                           f_digit = f_digit ? 0 : 1;
                                       } else {
                                           edit_set_color_buffer();
                                           ex++;
                                       }
                                       if( ex > bytes_per_row - 1 && ey == scrn.lines - 3) {
                                           ex = 0;
                                           file_next_line();
                                           set_editchange_colors();
                                           continue;
                                       }
                                       continue;
                    case VC_KEY_LEFT : if( !tab_flag) {
                                           if( !f_digit) {
                                               edit_set_color_buffer();
                                               ex--;
                                           }
                                           f_digit = f_digit ? 0 : 1;
                                       } else {
                                           edit_set_color_buffer();
                                           ex--;
                                       }
                                       if( ex < 0 && ey == 0) {
                                           ex = bytes_per_row - 1;
                                           file_prev_line();
                                           set_editchange_colors();
                                           continue;
                                       }
                                       continue;
                    case VC_KEY_HOME : file_go_home();
                                       set_editchange_colors();
                                       ex = ey = 0;
                                       continue;
                    case VC_KEY_END  : file_go_end();
                                       set_editchange_colors();
                                       ex = ey = 0;
                                       continue;
                    case VC_KEY_PGUP : file_prev_block();
                                       set_editchange_colors();
                                       ex = ey = 0;
                                       continue;
                    case VC_KEY_PGDN : file_next_block();
                                       set_editchange_colors();
                                       ex = ey = 0;
                                       continue;
                    default          : continue;
            }
        } /* end if( c == 27) */

        edit_the_buffer( c);

 } /* end while( 1) */

}

static void set_edit_fkeys( void)
{
 int sp, x, y;

 sp = ( scrn.cols / 10) - 8;
 y = scrn.lines - 1;

 term_color( VC_COLOR_BACKCYAN);
 raw_puts( 0, scrn.lines - 1, scrn.cols, replicate( ' ', scrn.cols));

 x  = 2;
 x += 8 + sp; raw_puts( x, y, 4, "SAVE"   ); /* F2  */
 x += 8 + sp; raw_puts( x, y, 6, "EDMASK" ); /* F3  */
 x += 8 + sp; raw_puts( x, y, 3, "XOR"    ); /* F4  */
 x += 8 + sp; raw_puts( x, y, 2, "OR"     ); /* F5  */
 x += 8 + sp; raw_puts( x, y, 3, "AND"    ); /* F6  */
 x += 8 + sp; raw_puts( x, y, 3, "NEG"    ); /* F7  */
 x += 8 + sp; raw_puts( x, y, 3, "ADD"    ); /* F8  */
 x += 8 + sp; raw_puts( x, y, 3, "SUB"    ); /* F9  */
 x += 8 + sp; raw_puts( x, y, 6, "CANCEL" ); /* F10 */

 term_color( VC_LIGHTGRAY );

 x  = 0;      raw_puts( x, y, 2, " 1");
 x += 8 + sp; raw_puts( x, y, 2, " 2");
 x += 8 + sp; raw_puts( x, y, 2, " 3");
 x += 8 + sp; raw_puts( x, y, 2, " 4");
 x += 8 + sp; raw_puts( x, y, 2, " 5");
 x += 8 + sp; raw_puts( x, y, 2, " 6");
 x += 8 + sp; raw_puts( x, y, 2, " 7");
 x += 8 + sp; raw_puts( x, y, 2, " 8");
 x += 8 + sp; raw_puts( x, y, 2, " 9");
 x += 8 + sp; raw_puts( x, y, 2, "10");
}

static void set_all_xy( void)
{
 /* The hex numbers begin at ey + 1 */
 hy = ey + 1;
 hx = 3.25 * ex + 12;

 /*
  *  Q: Where the f*** you get "hx = 3.25 * ex + 12" ???????????????
  *  A: Simple!
  *
  *  01234567890123456789012345678901234567890123456789012345678901
  *
  *  0000000000  68 65 6C 6C  6F 20 77 6F  72 6C 64 20  21 21 21 21  hello world
  *  0000000010  21 21 21 21  21 21 0A 0A  0A 0A       |             !!!!!!
  *              |              |__|__|   |____________|
  *              |____the hex      |                   |
  *                   numbers      |                   |
  *                   begin at     |                   |
  *                   ex = 12      |               at ex / 4 we
  *                                |               add one space
  *                             they are
  *                             separated
  *                             by ex * 2
  *
  *  Then we have this formula:   hx = ex + 12 + ( ex * 2) + ( ex / 4);
  *  We could render it this way: hx = ex + (2 * ex) + (1/4 * ex) + 12
  *  We simplify:                 hx = (1 + 2 + 1/4) * ex + 12
  *  Solution:                    hx = 3.25 * ex + 12
  *
  */

 ay = hy;
 ax = ex + scrn.cols - bytes_per_row;

 posxyh = 4 + 2 *( hy * scrn.cols + hx);
 posxya = 4 + 2 *( ay * scrn.cols + ax);
}

/*
 * Position the cursor and set the color of the hex
 * numbers
 */
static void edit_set_colorxy( int attr)
{
 char buffer[ 16];
 int co2 = scrn.cols * 2;
 int posh, posa;

 set_all_xy();

 posh = posxyh - (4 + co2) + co2;
 posa = posxya - (4 + co2) + co2;

 term_color( attr);

 buffer[ 0] = restore_buffer[ posh    ];
 buffer[ 1] = restore_buffer[ posh + 2];

 raw_puts( hx, hy, 2, buffer);

 buffer[ 0] = restore_buffer[ posa    ];

 raw_putc( ax, ay, buffer[ 0]);

 if( tab_flag)
     term_goto( ax, ay);
 else {
     if( f_digit) 
         term_goto( hx + 1, hy);
     else
         term_goto( hx, hy);
 }
}

void set_char_colorxy( char byte, int attr)
{
 int co2 = scrn.cols * 2;
 int posh, posa;

 set_all_xy();

 posh = posxyh - (4 + 2 * scrn.cols);
 posa = posxya - (4 + 2 * scrn.cols);

 obuffer[ posh    ] = restore_buffer[ posh     + co2 ] = hex_trans_byte1[ (unsigned char) byte];
 obuffer[ posh + 1] = restore_buffer[ posh + 1 + co2 ] = attr;
 obuffer[ posh + 2] = restore_buffer[ posh + 2 + co2 ] = hex_trans_byte2[ (unsigned char) byte];
 obuffer[ posh + 3] = restore_buffer[ posh + 3 + co2 ] = attr;

 obuffer[ posa    ] = restore_buffer[ posa     + co2 ] = byte > 31 && byte < 127 ? byte : '.';
 obuffer[ posa + 1] = restore_buffer[ posa + 1 + co2 ] = attr;
}

static void edit_the_buffer( char c)
{

 if( ( ey * bytes_per_row + ex) >= bread) {
     term_beep();
     return;
 }

 if( tab_flag) { /* ascii edit */
     edit_set_charxy_ascii( c);
 } else {        /* hex edit   */
     if( isxdigit( c))
         edit_set_charxy_hex( c);
     else
         term_beep();
 }

}

static void edit_set_charxy_ascii( char c)
{
 struct ll_edit *node = ll_edit_head;
 struct ll_edit *tmp  = ll_edit_head;
 char buf[ 16];
 int pos;
 int temp;

 set_all_xy();

 pos = ey * bytes_per_row + ex;

 while( node != NULL) {
        if( node->filepos < ( fpos + pos)) {
            tmp  = node;
            node = node->next;
        } else
            break;
 }

 temp = (unsigned char) c;

 term_color( VC_WHITE);

 buf[ 0] = hex_trans_byte1[ temp];
 buf[ 1] = hex_trans_byte2[ temp];

 raw_puts( hx, hy, 2, buf);

 raw_putc( ax, ay, c > 31 && c < 127 ? c : '.');

 if( hbuffer[ pos] != c) {

     if( node == NULL)
         ll_edit_add( c, fpos + pos, tmp);
     else 
         if( node->filepos == ( fpos + pos)){
             node->byte = c;
         }
         else
             ll_edit_add( c, fpos + pos, tmp);

     edit_set_colorxy( VC_BG_LIGHTGRAY);
     edit_set_color_in_buffer( VC_BG_LIGHTGRAY);
 } else {
     if( node != NULL) {
         if( node->filepos == ( fpos + pos)) {
             tmp->next = node->next;
             if( ll_edit_head == node)
                 ll_edit_head = tmp->next;
             free( node);
         }
     }
     edit_set_colorxy( VC_COLOR_BACKBLUE);
     edit_set_color_in_buffer( VC_COLOR_BACKBLUE);
 }

 ex++;
 if( ex > bytes_per_row - 1 && ey == scrn.lines - 3) {
     ex = 0;
     file_next_line();
     set_editchange_colors();
 }

}

static void edit_set_charxy_hex( char c)
{
 struct ll_edit *node = ll_edit_head;
 struct ll_edit *tmp  = ll_edit_head;
 char buf[ 16];
 int temp;
 int pos;
 int b1, b2;

 set_all_xy();

 pos = ey * bytes_per_row + ex;

 while( node != NULL) {
        if( node->filepos < ( fpos + pos)) {
            tmp  = node;
            node = node->next;
        } else
            break;
 }

 term_color( VC_WHITE);

 if( node == NULL) {
     buf[ 0] = hex_trans_byte1[ (unsigned char) hbuffer[ pos]];
     buf[ 1] = hex_trans_byte2[ (unsigned char) hbuffer[ pos]];
 } else {
     if( node->filepos == ( fpos + pos)) {
         buf[ 0] = hex_trans_byte1[ (unsigned char) node->byte];
         buf[ 1] = hex_trans_byte2[ (unsigned char) node->byte];
     } else {
         buf[ 0] = hex_trans_byte1[ (unsigned char) hbuffer[ pos]];
         buf[ 1] = hex_trans_byte2[ (unsigned char) hbuffer[ pos]];
     }
 }

 c = toupper( c);

 if( f_digit)
     buf[ 1] = c;
 else
     buf[ 0] = c;

 b1 = b162bin( buf, 0);
 b2 = b162bin( buf, 1);

 temp = b1 * 16 + b2;

 raw_puts( hx, hy, 2, buf);
 
 buf[ 0] = temp > 31 && temp < 127 ? temp : '.';

 raw_putc( ax, ay, buf[ 0]);

 if( hbuffer[ pos] != (char) temp) {
     if( node == NULL)
         ll_edit_add( temp, fpos + pos, tmp);
     else
         if( node->filepos == ( fpos + pos))
             node->byte = temp;
         else
             ll_edit_add( temp, fpos + pos, tmp);

     edit_set_colorxy( VC_BG_LIGHTGRAY);
     edit_set_color_in_buffer( VC_BG_LIGHTGRAY);
 } else {
     if( node != NULL) {
         if( node->filepos == ( fpos + pos)) {
             tmp->next = node->next;
             if( ll_edit_head == node)
                 ll_edit_head = tmp->next;
             free( node);
         }
     }
     edit_set_colorxy( VC_COLOR_BACKBLUE);
     edit_set_color_in_buffer( VC_COLOR_BACKBLUE);
 }

 if( f_digit) {
     ex++;
     if( ex > bytes_per_row - 1 && ey == scrn.lines - 3) {
         ex = 0;
         file_next_line();
         set_editchange_colors();
     }
 }

 f_digit = f_digit ? 0 : 1;
}

static void set_editchange_colors( void)
{
 struct ll_edit *node = ll_edit_head;
 int tmpx, tmpy;
 int bytes;

 memset( cbuffer, VC_COLOR_BACKBLUE, ( scrn.lines - 2 ) * scrn.cols);

 while( node != NULL) {
        if( node->filepos < fpos)
            node = node->next;
        else
            break;
 }

 if( node != NULL) {
     tmpx = ex;
     tmpy = ey;

     bytes = node->filepos - fpos;

     while( bytes < bread && node != NULL && node->filepos <= (fpos + bytes)) {

          ey = bytes / bytes_per_row;
          ex = - ey * bytes_per_row + bytes;

          set_char_colorxy( node->byte, VC_BG_LIGHTGRAY);
          edit_set_color_in_buffer( VC_BG_LIGHTGRAY);
          node = node->next;
          ex++;
          if( ex > bytes_per_row - 1) {
              ex = 0;
              ey++;
          }
          if( node != NULL)
              bytes = node->filepos - fpos;
     }

     ex = tmpx;
     ey = tmpy;
 }

 raw_flush();
}

static void edit_set_mask( void)
{
 char emask[ 256];

 raw_save_screen();

 memset( &emask, 0, sizeof( emask));

 sprintf( emask, "0x%X", edit_mask);

 raw_dialog_box( "Enter edit mask: ", emask, 38, VC_COLOR_BACKCYAN, VC_WHITE);

 term_goto( (scrn.cols - 38) / 2, scrn.lines / 2);
 key_scanf( emask, 38);
 term_unhide_cursor();

 if( !strncasecmp( emask, "0x", 2))
     sscanf( emask, "%x", &edit_mask);
 else
     edit_mask = atoi( emask);

 raw_load_screen();
}

static void edit_bitops( int bitop)
{
 struct ll_edit *node = ll_edit_head;
 struct ll_edit *tmp  = ll_edit_head;
 char buf[ 16];
 char c = 0;
 int pos;
 int temp;

 set_all_xy();

 pos = ey * bytes_per_row + ex;

 if( pos >= bread) {
     term_beep();
     return;
 }

 while( node != NULL) {
        if( node->filepos < ( fpos + pos)) {
            tmp  = node;
            node = node->next;
        } else
            break;
 }

 if( node != NULL && node->filepos == ( fpos + pos))
     switch( bitop) {
             case EDIT_BITOP_XOR: c =  node->byte ^ edit_mask;
                                  break;
             case EDIT_BITOP_OR : c =  node->byte | edit_mask;
                                  break;
             case EDIT_BITOP_AND: c =  node->byte & edit_mask;
                                  break;
             case EDIT_BITOP_NEG: c = ~node->byte;
                                  break;
             case EDIT_BITOP_ADD: c = node->byte;
                                  c++; 
                                  break;
             case EDIT_BITOP_SUB: c = node->byte;
                                  c--;
                                  break;
     }
 else
     switch( bitop) {
             case EDIT_BITOP_XOR: c =  hbuffer[ pos] ^ edit_mask;
                                  break;
             case EDIT_BITOP_OR : c =  hbuffer[ pos] | edit_mask;
                                  break;
             case EDIT_BITOP_AND: c =  hbuffer[ pos] & edit_mask;
                                  break;
             case EDIT_BITOP_NEG: c = ~hbuffer[ pos];
                                  break;
             case EDIT_BITOP_ADD: c = hbuffer[ pos];
                                  c++;
                                  break;
             case EDIT_BITOP_SUB: c = hbuffer[ pos];
                                  c--;
                                  break;
     }

 temp = (unsigned char) c;

 term_color( VC_WHITE);
 buf[ 0] = hex_trans_byte1[ temp];
 buf[ 1] = hex_trans_byte2[ temp];

 raw_puts( hx, hy, 2, buf);

 raw_putc( ax, ay, c > 31 && c < 127 ? c : '.');

 if( hbuffer[ pos] != c) {

     if( node == NULL)
         ll_edit_add( c, fpos + pos, tmp);
     else
         if( node->filepos == ( fpos + pos)){
             node->byte = c;
         }
         else
             ll_edit_add( c, fpos + pos, tmp);

     edit_set_colorxy( VC_BG_LIGHTGRAY);
     edit_set_color_in_buffer( VC_BG_LIGHTGRAY);
 } else {
     if( node != NULL) {
         if( node->filepos == ( fpos + pos)) {
             tmp->next = node->next;
             if( ll_edit_head == node)
                 ll_edit_head = tmp->next;
             free( node);
         }
     }
     edit_set_colorxy( VC_COLOR_BACKBLUE);
     edit_set_color_in_buffer( VC_COLOR_BACKBLUE);
 }

 ex++;
 if( ex > bytes_per_row - 1 && ey == scrn.lines - 3) {
     ex = 0;
     file_next_line();
     set_editchange_colors();
 }

}

static int edit_ask_save( int pos)
{
 int i;

 if( ll_edit_head == NULL)
     return -1;

 i = raw_ask( "The file was modified, save the changes?", " Yes ", " No ", " Cancel ", pos);

 if( i == 0) {
     edit_save_data();
     return -1;
 } else 
     if( i == 1)
         return -1;

 return 0;
}

static void edit_save_data( void)
{
 struct ll_edit *node = ll_edit_head;
 struct ll_edit *temp = ll_edit_head;

 raw_save_screen();

 raw_show_message( " Saving... ", 0);

 while( node != NULL) {
        lseek( fd, node->filepos, SEEK_SET);
        write( fd, &node->byte, 1);
        temp = node;
        node = node->next;
        free( temp);
 }

 lseek( fd, (long) fpos, SEEK_SET);

 ll_edit_head = NULL;

 raw_load_screen();
}

