/*
   Help functions
   --------------------------------------------------------------------
   VCHE RAW - Virtual Console Hex Editor Run Anywhere version

   Copyright (C) 1998, 1999 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

static char *meaning[] = { 
 " NUL "," SOH "," STX "," ETX "," EOT "," ENQ "," ACK ",
 " BEL "," BS  "," HT  "," LF  "," VT  "," FF  "," CR  ",
 " SO  "," SI  "," DLE "," DC1 "," DC2 "," DC3 "," DC4 ",
 " NAK "," SYN "," ETB "," CAN "," EM  "," SUB "," ESC ",
 " FS  "," GS  "," RS  "," US  ","SPACE",
};

/*
 * Function prototypes
 */
static char *int_to_escape( int c);
static char *int_to_binary( int c);

/*
 * VCHE main help
 */
void help( void)
{
 char hlptxt[ 2048];

 sprintf( hlptxt, "\n  VCHE RAW - Virtual Console Hex Editor Run AnyWhere version\n\
\n\
  Release %s\n\
\n\
  If you need help, type \"man vche\" at the shell prompt.\n\n\
  It is the RAW version of VCHE, to see all the 256 characters\n\
  run \"vche\" instead.\n\n\
  If you have found a bug, have problems or comments, mail\n\
  me to <diego@grigna.com>\n", VCHE_VERSION);

 raw_show_screentext( hlptxt, " VCHE-RAW HELP ");
}

void show_gnu_license( void)
{
 char *gnugpl = "\n  VCHE RAW - Virtual Console Hex Editor Run AnyWhere version\n\
  Copyright (C) 1998, 1999 Diego Javier Grigna <diego@grigna.com>\n\
\n\
  This program is free software; you can redistribute it and/or modify\n\
  it under the terms of the GNU General Public License as published by\n\
  the Free Software Foundation; either version 2 of the License, or\n\
  (at your option) any later version.\n\
\n\
  This program is distributed in the hope that it will be useful,\n\
  but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
  GNU General Public License for more details.\n\
\n\
  You should have received a copy of the GNU General Public License\n\
  along with this program; if not, write to the Free Software\n\
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n\n";

 raw_show_screentext( gnugpl, " GNU General Public License ");
}

/*
 * Shows the ascii table and let people navigate it.
 */
void ascii_table( void)
{
 char buf[ 128];
 char *bottom_line = "Use         to move           ENTER or F10 to QUIT";
 int slines = scrn.lines - 9; /* Number of lines to show */
 int endpos;
 int pos = 0;
 int ll;
 int i, j;
 char c;

 endpos = 256 - slines;
 if( endpos < 0) endpos = 0;

 raw_save_screen();

 raw_box( 0, 0, scrn.cols, scrn.lines - 1, VC_COLOR_BACKCYAN);
 raw_do_title( " ASCII Table ");
 raw_put_bottom_line( bottom_line);

 ll = ( scrn.cols - strlen( bottom_line)) / 2;

 term_enable_graf_map();
 raw_putc( ll + 24, scrn.lines - 1, RAW_LINE_VERT);
 raw_putc( ll +  4, scrn.lines - 1, RAW_UARROW   );
 raw_putc( ll +  6, scrn.lines - 1, RAW_DARROW   );
 raw_putc( ll +  8, scrn.lines - 1, RAW_RARROW   );
 raw_putc( ll + 10, scrn.lines - 1, RAW_LARROW   );
 term_disable_graf_map();

 term_color( VC_COLOR_BACKCYAN);

 raw_puts( 9, 3, 59, "Dec   Oct   Hex   Character   Meaning   C Escape     Binary");

 ll = scrn.lines - 4;

 term_enable_graf_map();

 raw_putc(  7,  2, RAW_LINE_TOPLEFT    );
 raw_putc( 71,  2, RAW_LINE_TOPRIGHT   );
 raw_putc(  7,  4, RAW_LTEE            );
 raw_putc( 71,  4, RAW_RTEE            );
 raw_putc(  7, ll, RAW_LINE_BOTTOMLEFT );
 raw_putc( 71, ll, RAW_LINE_BOTTOMRIGHT);

 for( i = 1; i < 64; i++) {
      raw_putc( 7 + i,  2, RAW_LINE_HORIZ);
      raw_putc( 7 + i,  4, RAW_LINE_HORIZ);
      raw_putc( 7 + i, ll, RAW_LINE_HORIZ);
 }

 for( i = 5; i < scrn.lines - 4; i++) {
      raw_putc(  7, i, RAW_LINE_VERT);
      raw_putc( 71, i, RAW_LINE_VERT);
 }

 raw_putc( 13, 2, RAW_TTEE);
 raw_putc( 19, 2, RAW_TTEE);
 raw_putc( 25, 2, RAW_TTEE);
 raw_putc( 37, 2, RAW_TTEE);
 raw_putc( 47, 2, RAW_TTEE);
 raw_putc( 58, 2, RAW_TTEE);

 raw_putc(  7, 3, RAW_LINE_VERT);
 raw_putc( 13, 3, RAW_LINE_VERT);
 raw_putc( 19, 3, RAW_LINE_VERT);
 raw_putc( 25, 3, RAW_LINE_VERT);
 raw_putc( 37, 3, RAW_LINE_VERT);
 raw_putc( 47, 3, RAW_LINE_VERT);
 raw_putc( 58, 3, RAW_LINE_VERT);
 raw_putc( 71, 3, RAW_LINE_VERT);

 raw_putc( 13, 4, RAW_CROSS);
 raw_putc( 19, 4, RAW_CROSS);
 raw_putc( 25, 4, RAW_CROSS);
 raw_putc( 37, 4, RAW_CROSS);
 raw_putc( 47, 4, RAW_CROSS);
 raw_putc( 58, 4, RAW_CROSS);

 raw_putc( 13, ll, RAW_BTEE);
 raw_putc( 19, ll, RAW_BTEE);
 raw_putc( 25, ll, RAW_BTEE);
 raw_putc( 37, ll, RAW_BTEE);
 raw_putc( 47, ll, RAW_BTEE);
 raw_putc( 58, ll, RAW_BTEE);

 term_disable_graf_map();

 j = 5;

 while( 1) {
        if( pos > endpos) {
            term_disable_graf_map();
            term_beep();
            pos = endpos;
        }

        if( pos < 0) {
            term_disable_graf_map();
            term_beep();
            pos = 0;
        }

        if( pos == 0) {
            term_color( VC_COLOR_BACKCYAN);
            /* I have to put it here, because a bug or feature of rxvt */
            term_enable_graf_map(); 
            raw_putc( 10,  4, RAW_LINE_HORIZ );

            term_color( VC_COLOR_ATTENTION);
            term_enable_graf_map();
            raw_putc( 10, scrn.lines - 4, RAW_DARROW);
        } else 
            if( pos == endpos) {
                term_color( VC_COLOR_ATTENTION);
                term_enable_graf_map(); 
                raw_putc( 10,  4, RAW_UARROW);

                term_color( VC_COLOR_BACKCYAN);
                term_enable_graf_map(); 
                raw_putc( 10, scrn.lines - 4, RAW_LINE_HORIZ);
            } else {
                term_color( VC_COLOR_ATTENTION);
                term_enable_graf_map(); 
                raw_putc( 10,              4, RAW_UARROW);
                raw_putc( 10, scrn.lines - 4, RAW_DARROW);
            }

        for( i = pos; i < pos + slines; i++) {
             if( j > scrn.lines - 5)
                 j = 5;

             if( i < 256) {
                 sprintf( buf, "%3d   %03o   %02X        %c        %s      %s      %s"
                               , i, i, i, ( i > 31) && ( i < 127) ? i : '.'
                               , (i == 127) ? " DEL ": i <= 32 ? meaning[ i]: "     "
                               , int_to_escape( i)
                               , int_to_binary( i));
                 term_color( VC_COLOR_BACKCYAN);
                 term_disable_graf_map();
                 raw_puts( 9, j, 60, buf);
             }

             term_enable_graf_map();
             raw_putc( 13, j, RAW_LINE_VERT);
             raw_putc( 19, j, RAW_LINE_VERT);
             raw_putc( 25, j, RAW_LINE_VERT);
             raw_putc( 37, j, RAW_LINE_VERT);
             raw_putc( 47, j, RAW_LINE_VERT);
             raw_putc( 58, j, RAW_LINE_VERT);
             j++;
        }

        c = getc( stdin);

        if( c == '\n' || c == 'q' || c == 'Q') {
            term_disable_graf_map();
            raw_load_screen();
            return;
        }

        if( c == 27) {
            switch( key_process()) {
                    case VC_KEY_UP   :
                    case VC_KEY_LEFT : pos--;
                                       break;
                    case VC_KEY_DOWN :
                    case VC_KEY_RIGHT: pos++;
                                       break;
                    case VC_KEY_PGUP : pos -= slines;
                                       break;
                    case VC_KEY_PGDN : pos += slines;
                                       break;
                    case VC_KEY_HOME : pos = 0;
                                       break;
                    case VC_KEY_END  : pos = endpos;
                                       break;
                    case VC_KEY_F10  : term_disable_graf_map();
                                       raw_load_screen();
                                       return;
            }
        } /* end if( c == 27) */
 }

}

/*
 * Return the C language escape char
 */
static char *int_to_escape( int c)
{

 switch( c) {
         case  0: return "'\\0'";
         case  7: return "'\\a'";
         case  8: return "'\\b'";
         case  9: return "'\\t'";
         case 10: return "'\\n'";
         case 11: return "'\\v'";
         case 12: return "'\\f'";
         case 13: return "'\\r'";
         case 92: return "'\\\\'";
         default: return "    ";
 }

}

/*
 * Convert from integer to binary
 */
static char *int_to_binary( int c)
{
 static char bbuf[ 16];
 register char *bp = bbuf + 8;

 do {
     *--bp = (c % 2) ? '1' : '0';
 } while( (c /= 2) != 0);

 while( bp != bbuf)
        *--bp = '0';

 return bbuf;
}

