/*
   Help functions
   --------------------------------------------------------------------
   VCHE - Virtual Console Hex Editor

   Copyright (C) 1998, 1999 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

static char *meaning[] = { 
 " NUL "," SOH "," STX "," ETX "," EOT "," ENQ "," ACK ",
 " BEL "," BS  "," HT  "," LF  "," VT  "," FF  "," CR  ",
 " SO  "," SI  "," DLE "," DC1 "," DC2 "," DC3 "," DC4 ",
 " NAK "," SYN "," ETB "," CAN "," EM  "," SUB "," ESC ",
 " FS  "," GS  "," RS  "," US  ","SPACE",
};

/*
 * Function prototypes
 */
static char *int_to_escape( int c);
static char *int_to_binary( int c);

/*
 * VCHE main help
 */
void help( void)
{
 char hlptxt[ 2048];

 sprintf( hlptxt, "\n  VCHE - Virtual Console Hex Editor\n\
\n\
  Release %s\n\
\n\
  If you need help, type \"man vche\" at the shell prompt.\n\n\
  If you have found a bug, have problems or comments, mail\n\
  me to <diego@grigna.com>\n", VCHE_VERSION);

 vc_show_screentext( hlptxt, " VCHE HELP ");
}

void show_gnu_license( void)
{
 char *gnugpl = "\n  VCHE - Virtual Console Hex Editor\n\
  Copyright (C) 1998, 1999 Diego Javier Grigna <diego@grigna.com>\n\
\n\
  This program is free software; you can redistribute it and/or modify\n\
  it under the terms of the GNU General Public License as published by\n\
  the Free Software Foundation; either version 2 of the License, or\n\
  (at your option) any later version.\n\
\n\
  This program is distributed in the hope that it will be useful,\n\
  but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
  GNU General Public License for more details.\n\
\n\
  You should have received a copy of the GNU General Public License\n\
  along with this program; if not, write to the Free Software\n\
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n\n";

 vc_show_screentext( gnugpl, " GNU General Public License ");
}

/*
 * Shows the ascii table and let people navigate it.
 */
void ascii_table( void)
{
 char buf[ 128];
 int slines = scrn.lines - 9; /* Number of lines to show */
 int endpos;
 int pos = 0;
 int i, j;
 char c;

 endpos = 256 - slines;
 if( endpos < 0) endpos = 0;

 vc_save_screen();

 vc_box( 0, 0, scrn.cols, scrn.lines - 1, VC_COLOR_BACKCYAN);
 vc_do_title( " ASCII Table ");
 vc_put_bottom_line( "Use \x18 \x19 \x1A \x1B to move          ENTER or F10 to QUIT" );

 vc_puts( 7,  2, 65, "Ŀ");
 vc_puts( 7,  3, 65, " Dec  Oct  Hex  Character  Meaning  C Escape    Binary   ");
 vc_puts( 7,  4, 65, "Ĵ");
 vc_puts( 7, scrn.lines - 4, 65, "");

 j = 5;

 while( 1) {
        if( pos > endpos) {
            term_beep();
            pos = endpos;
        }

        if( pos < 0) {
            term_beep();
            pos = 0;
        }

        if( pos == 0) {
            vc_color_putc( 10,  4, ''   , VC_COLOR_BACKCYAN );
            vc_color_putc( 10, scrn.lines - 4, '\x1F', VC_COLOR_ATTENTION);
        } else 
            if( pos == endpos) {
                vc_color_putc( 10,  4, '\x1E', VC_COLOR_ATTENTION);
                vc_color_putc( 10, scrn.lines - 4, ''   , VC_COLOR_BACKCYAN );
            } else {
                vc_color_putc( 10,  4, '\x1E', VC_COLOR_ATTENTION);
                vc_color_putc( 10, scrn.lines - 4, '\x1F', VC_COLOR_ATTENTION);
            }

        for( i = pos; i < pos + slines; i++) {
             if( j > scrn.lines - 5)
                 j = 5;
             if( i < 256)
                 sprintf( buf, " %3d  %03o  %02X       %c       %s     %s     %s  ", i, i, i, i, (i == 127) ? " DEL ": i <= 32 ? meaning[ i]: "     ", int_to_escape( i), int_to_binary( i));
             else
                 sprintf( buf, "                                                  ");

                 vc_puts( 7, j, 65, buf);
             j++;
        }

        c = getc( stdin);

        if( c == '\n' || c == 'q' || c == 'Q') {
            vc_load_screen();
            return;
        }

        if( c == 27) {
            switch( key_process()) {
                    case VC_KEY_UP   :
                    case VC_KEY_LEFT : pos--;
                                       break;
                    case VC_KEY_DOWN :
                    case VC_KEY_RIGHT: pos++;
                                       break;
                    case VC_KEY_PGUP : pos -= slines;
                                       break;
                    case VC_KEY_PGDN : pos += slines;
                                       break;
                    case VC_KEY_HOME : pos = 0;
                                       break;
                    case VC_KEY_END  : pos = endpos;
                                       break;
                    case VC_KEY_F10  : vc_load_screen();
                                       return;
            }
        } /* end if( c == 27) */
 }

}

/*
 * Return the C language escape char
 */
static char *int_to_escape( int c)
{

 switch( c) {
         case  0: return "'\\0'";
         case  7: return "'\\a'";
         case  8: return "'\\b'";
         case  9: return "'\\t'";
         case 10: return "'\\n'";
         case 11: return "'\\v'";
         case 12: return "'\\f'";
         case 13: return "'\\r'";
         case 92: return "'\\\\'";
         default: return "    ";
 }

}

/*
 * Convert from integer to binary
 */
static char *int_to_binary( int c)
{
 static char bbuf[ 16];
 register char *bp = bbuf + 8;

 do {
     *--bp = (c % 2) ? '1' : '0';
 } while( (c /= 2) != 0);

 while( bp != bbuf)
        *--bp = '0';

 return bbuf;
}

